@@ -2,6 +2,7 @@
 
 #define ERR(fmt, args...) fprintf(stderr, fmt, ## args)
 #define DBG2(fmt, args...) fprintf(stderr, "%s:%s:%-5d: " fmt, __FILE__,  __FUNCTION__, __LINE__, ## args)
+#define DBGE2(fmt, args...)       fprintf(stderr, fmt, ## args)
 #define ENTER2          do { fprintf(stderr, "%s:%s:%-5d: ENTER\n",  __FILE__,__FUNCTION__, __LINE__); } while(0)
 #define RET2(args...)   do { fprintf(stderr, "%s:%s:%-5d: RETURN\n",  __FILE__,__FUNCTION__, __LINE__);\
 return args; } while(0)
@@ -13,6 +14,7 @@ enum { LOG_NONE, LOG_ERR, LOG_WARN, LOG_INFO, LOG_ALL };
 #define RET(args...)   do { fprintf(stderr, "%s:%s:%-5d: RETURN\n", __FILE__, __FUNCTION__, __LINE__);\
 return args; } while(0)
 #define DBG(fmt, args...) fprintf(stderr, "%s:%s:%-5d: " fmt,  __FILE__,__FUNCTION__, __LINE__, ## args)
+#define DBGE(fmt, args...)       fprintf(stderr, fmt, ## args)
 #define LOG(level, fmt, args...) fprintf(stderr, fmt, ## args)
 
 #else
@@ -21,6 +23,7 @@ extern int log_level;
 #define ENTER         do {  } while(0)
 #define RET(args...)   return args
 #define DBG(fmt, args...)   do {  } while(0)
+#define DBGE(fmt, args...)   do {  } while(0)
 #define LOG(level, fmt, args...) do { if (level <= log_level) fprintf(stderr, fmt, ## args); } while(0)
 
 #endif
