@@ -9,6 +9,7 @@
 #include <locale.h>
 #include <string.h>
 #include <signal.h>
+#include <time.h>
 
 #include "plugin.h"
 #include "panel.h"
@@ -24,6 +25,7 @@ gchar *cprofile = "default";
 int config = 0;
 FbEv *fbev;
 gint force_quit = 0;
+
 //#define DEBUG
 #include "dbg.h"
 
@@ -62,37 +64,41 @@ panel_set_wm_strut(panel *p)
         data[i] = p->aw;
         data[4 + i*2] = p->ay;
         data[5 + i*2] = p->ay + p->ah;
+        if (p->autohide) data[i] = p->height_when_hidden;
         break;
     case EDGE_RIGHT:
         i = 1;
         data[i] = p->aw;
         data[4 + i*2] = p->ay;
         data[5 + i*2] = p->ay + p->ah;
+        if (p->autohide) data[i] = p->height_when_hidden;
         break;
     case EDGE_TOP:
         i = 2;
         data[i] = p->ah;
         data[4 + i*2] = p->ax;
         data[5 + i*2] = p->ax + p->aw;
+        if (p->autohide) data[i] = p->height_when_hidden;
         break;
     case EDGE_BOTTOM:
         i = 3;
         data[i] = p->ah;
         data[4 + i*2] = p->ax;
         data[5 + i*2] = p->ax + p->aw;
+        if (p->autohide) data[i] = p->height_when_hidden;
         break;
     default:
         ERR("wrong edge %d. strut won't be set\n", p->edge);
         RET();
-    }		
+    }
     DBG("type %d. width %d. from %d to %d\n", i, data[i], data[4 + i*2], data[5 + i*2]);
-                
+
     /* if wm supports STRUT_PARTIAL it will ignore STRUT */
-    XChangeProperty(GDK_DISPLAY(), p->topxwin, a_NET_WM_STRUT_PARTIAL, 
+    XChangeProperty(GDK_DISPLAY(), p->topxwin, a_NET_WM_STRUT_PARTIAL,
           XA_CARDINAL, 32, PropModeReplace,  (unsigned char *) data, 12);
     /* old spec, for wms that do not support STRUT_PARTIAL */
-    XChangeProperty(GDK_DISPLAY(), p->topxwin, a_NET_WM_STRUT, 
-          XA_CARDINAL, 32, PropModeReplace,  (unsigned char *) data, 4); 
+    XChangeProperty(GDK_DISPLAY(), p->topxwin, a_NET_WM_STRUT,
+          XA_CARDINAL, 32, PropModeReplace,  (unsigned char *) data, 4);
 
     RET();
 }
@@ -127,23 +133,23 @@ panel_event_filter(GdkXEvent *xevent, GdkEvent *event, panel *p)
     DBG("win = 0x%x\n", ev->xproperty.window);
     if (ev->type != PropertyNotify )
         RET(GDK_FILTER_CONTINUE);
-    
+
     at = ev->xproperty.atom;
     win = ev->xproperty.window;
     DBG("win=%x at=%d\n", win, at);
     if (win == GDK_ROOT_WINDOW()) {
-	if (at == a_NET_CLIENT_LIST) {
+    if (at == a_NET_CLIENT_LIST) {
             DBG("A_NET_CLIENT_LIST\n");
             fb_ev_trigger(fbev, EV_CLIENT_LIST);
-	} else if (at == a_NET_CURRENT_DESKTOP) {
+    } else if (at == a_NET_CURRENT_DESKTOP) {
             DBG("A_NET_CURRENT_DESKTOP\n");
             p->curdesk = get_net_current_desktop();
             fb_ev_trigger(fbev, EV_CURRENT_DESKTOP);
-	} else if (at == a_NET_NUMBER_OF_DESKTOPS) {
+    } else if (at == a_NET_NUMBER_OF_DESKTOPS) {
             DBG("A_NET_NUMBER_OF_DESKTOPS\n");
             p->desknum = get_net_number_of_desktops();
             fb_ev_trigger(fbev, EV_NUMBER_OF_DESKTOPS);
-	} else if (at == a_NET_DESKTOP_NAMES) {
+    } else if (at == a_NET_DESKTOP_NAMES) {
             DBG("A_NET_DESKTOP_NAMES\n");
             fb_ev_trigger(fbev, EV_DESKTOP_NAMES);
         } else if (at == a_NET_ACTIVE_WINDOW) {
@@ -152,7 +158,7 @@ panel_event_filter(GdkXEvent *xevent, GdkEvent *event, panel *p)
         }else if (at == a_NET_CLIENT_LIST_STACKING) {
             DBG("A_NET_CLIENT_LIST_STACKING\n");
             fb_ev_trigger(fbev, EV_CLIENT_LIST_STACKING);
-	} else if (at == a_NET_WORKAREA) {
+    } else if (at == a_NET_WORKAREA) {
             DBG("A_NET_WORKAREA\n");
             p->workarea = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_WORKAREA, XA_CARDINAL, &p->wa_len);
             print_wmdata(p);
@@ -168,7 +174,7 @@ panel_event_filter(GdkXEvent *xevent, GdkEvent *event, panel *p)
  *         panel's handlers for GTK events          *
  ****************************************************/
 
-  
+
 static gint
 panel_delete_event(GtkWidget * widget, GdkEvent * event, gpointer data)
 {
@@ -181,7 +187,7 @@ panel_destroy_event(GtkWidget * widget, GdkEvent * event, gpointer data)
 {
     //panel *p = (panel *) data;
 
-    ENTER;  
+    ENTER;
     //if (!p->self_destroy)
     gtk_main_quit();
     RET(FALSE);
@@ -215,7 +221,7 @@ static gint
 panel_size_alloc(GtkWidget *widget, GtkAllocation *a, panel *p)
 {
     static gint x, y;
-    
+
     ENTER;
     DBG("gtk: size (%d, %d). pos (%d, %d)\n", a->width, a->height, a->x, a->y);
     DBG("our: size (%d, %d). pos (%d, %d)\n", p->aw, p->ah, p->ax, p->ay);
@@ -238,17 +244,6 @@ panel_size_alloc(GtkWidget *widget, GtkAllocation *a, panel *p)
     RET(TRUE);
 }
 
-static void
-panel_style_set(GtkWidget *widget, GtkStyle  *previous_style, panel *p)
-{
-    ENTER;
-    if (GTK_WIDGET_REALIZED(widget)) {
-        DBG("realized\n");
-        gtk_main_quit();
-    }
-    RET();
-}
-
 static  gboolean
 panel_configure_event (GtkWidget *widget, GdkEventConfigure *e, panel *p)
 {
@@ -265,7 +260,7 @@ panel_configure_event (GtkWidget *widget, GdkEventConfigure *e, panel *p)
     DBG("here\n");
     DBG("geom: size (%d, %d). pos (%d, %d)\n", e->width, e->height, e->x, e->y);
     RET(FALSE);
-    
+
 }
 
 
@@ -280,7 +275,7 @@ make_round_corners(panel *p)
     void (*box_pack)(GtkBox *, GtkWidget *, gboolean, gboolean, guint);
     gchar *s1, *s2;
 #define IMGPREFIX  PREFIX "/share/fbpanel/images/"
-    
+
     ENTER;
     if (p->edge == EDGE_TOP) {
         s1 = IMGPREFIX "top-left.xpm";
@@ -296,7 +291,7 @@ make_round_corners(panel *p)
         s2 = IMGPREFIX "bottom-right.xpm";
     } else
         RET();
-    
+
     box_new = (p->orientation == ORIENT_HORIZ) ? gtk_vbox_new : gtk_hbox_new;
     b1 = box_new(0, FALSE);
     gtk_widget_show(b1);
@@ -305,7 +300,7 @@ make_round_corners(panel *p)
 
     box_pack = (p->edge == EDGE_TOP || p->edge == EDGE_LEFT) ?
         gtk_box_pack_start : gtk_box_pack_end;
-     
+
     img = gtk_image_new_from_file(s1);
     gtk_widget_show(img);
     box_pack(GTK_BOX(b1), img, FALSE, FALSE, 0);
@@ -314,17 +309,77 @@ make_round_corners(panel *p)
     box_pack(GTK_BOX(b2), img, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(p->lbox), b1, FALSE, FALSE, 0);
     gtk_box_pack_end(GTK_BOX(p->lbox), b2, FALSE, FALSE, 0);
-    RET();      
+    RET();
 }
 
+
+static gboolean
+panel_leave_real(panel *p)
+{
+    static GdkDisplay *display;
+    static int count;
+    gint x, y;
+    
+    ENTER;
+    if (!display)
+        display = gdk_display_get_default();
+    
+    if (gdk_display_pointer_is_grabbed(display)) {
+        count = 0;
+        RET(TRUE);
+    }
+    gdk_display_get_pointer(display, NULL, &x, &y, NULL);
+    if ((p->cx <= x && x <= (p->cx + p->cw))
+          && (p->cy <= y && y <= (p->cy + p->ch))) {
+        count = 0;
+        RET(TRUE);
+    }
+    DBG("count=%d\n", count);
+    if (count++ == 0)
+        RET(TRUE);
+    gtk_widget_hide(p->lbox);
+    p->visible = HIDDEN;    
+    p->hide_tout = 0;
+    DBG("hide panel\n");
+    count = 0;
+    RET(FALSE);
+}
+
+
+
+
+static gboolean
+panel_enter (GtkImage *widget, GdkEventCrossing *event, panel *p)
+{
+    ENTER;
+    if (p->hide_tout)
+        RET(FALSE);
+
+    p->hide_tout = g_timeout_add(500, (GSourceFunc) panel_leave_real, p);
+    gtk_widget_show(p->lbox);
+    p->visible = VISIBLE;
+    DBG("show panel\n");
+    RET(TRUE);
+}
+
+static gboolean
+panel_drag_motion (GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time,
+      panel *p)
+{
+    ENTER;
+    panel_enter (NULL, NULL, p);
+    RET(TRUE);
+}
+
+
 void
 panel_start_gui(panel *p)
 {
     Atom state[3];
     XWMHints wmhints;
     guint32 val;
- 
-    
+
+
     ENTER;
 
     // main toplevel window
@@ -335,7 +390,9 @@ panel_start_gui(panel *p)
     gtk_window_set_title(GTK_WINDOW(p->topgwin), "panel");
     gtk_window_set_position(GTK_WINDOW(p->topgwin), GTK_WIN_POS_NONE);
     gtk_window_set_decorated(GTK_WINDOW(p->topgwin), FALSE);
-    
+    //GTK_WIDGET_UNSET_FLAGS (p->topgwin, GTK_CAN_FOCUS);
+    gtk_window_set_accept_focus(GTK_WINDOW(p->topgwin), FALSE);    
+   
     g_signal_connect(G_OBJECT(p->topgwin), "delete-event",
           G_CALLBACK(panel_delete_event), p);
     g_signal_connect(G_OBJECT(p->topgwin), "destroy-event",
@@ -348,13 +405,11 @@ panel_start_gui(panel *p)
           (GCallback) panel_configure_event, p);
     g_signal_connect (G_OBJECT (p->topgwin), "realize",
           (GCallback) panel_realize, p);
-    g_signal_connect (G_OBJECT (p->topgwin), "style-set",
-          (GCallback) panel_style_set, p);
-    
+  
     gtk_widget_realize(p->topgwin);
     //gdk_window_set_decorations(p->topgwin->window, 0);
     gtk_widget_set_app_paintable(p->topgwin, TRUE);
-    
+
     // background box all over toplevel
     p->bbox = gtk_bgbox_new();
     gtk_container_add(GTK_CONTAINER(p->topgwin), p->bbox);
@@ -362,7 +417,7 @@ panel_start_gui(panel *p)
     gtk_container_set_border_width(GTK_CONTAINER(p->bbox), 0);
     if (p->transparent) {
         p->bg = fb_bg_get_for_display();
-        gtk_bgbox_set_background(p->bbox, BG_ROOT, p->tintcolor, p->alpha);        
+        gtk_bgbox_set_background(p->bbox, BG_ROOT, p->tintcolor, p->alpha);
     }
 
     // main layout manager as a single child of background widget box
@@ -372,20 +427,20 @@ panel_start_gui(panel *p)
     gtk_widget_show(p->lbox);
     if (p->round_corners)
         make_round_corners(p);
- 
-    p->box = p->my_box_new(FALSE, p->spacing); 
+
+    p->box = p->my_box_new(FALSE, p->spacing);
     gtk_container_set_border_width(GTK_CONTAINER(p->box), 0);
     gtk_box_pack_start(GTK_BOX(p->lbox), p->box, TRUE, TRUE, 0);
     gtk_widget_show(p->box);
-      
+
     p->topxwin = GDK_WINDOW_XWINDOW(GTK_WIDGET(p->topgwin)->window);
     DBG("topxwin = %x\n", p->topxwin);
 
     /* the settings that should be done before window is mapped */
     wmhints.flags = InputHint;
     wmhints.input = 0;
-    XSetWMHints (GDK_DISPLAY(), p->topxwin, &wmhints); 
-#define WIN_HINTS_SKIP_FOCUS      (1<<0)	/* "alt-tab" skips this win */
+    XSetWMHints (GDK_DISPLAY(), p->topxwin, &wmhints);
+#define WIN_HINTS_SKIP_FOCUS      (1<<0)    /* "alt-tab" skips this win */
     val = WIN_HINTS_SKIP_FOCUS;
     XChangeProperty(GDK_DISPLAY(), p->topxwin,
           XInternAtom(GDK_DISPLAY(), "_WIN_HINTS", False), XA_CARDINAL, 32,
@@ -418,11 +473,38 @@ panel_start_gui(panel *p)
     XSelectInput (GDK_DISPLAY(), GDK_ROOT_WINDOW(), PropertyChangeMask);
     gdk_window_add_filter(gdk_get_default_root_window (), (GdkFilterFunc)panel_event_filter, p);
 
+
+
+    if (p->autohide) {
+        gtk_widget_add_events(p->topgwin, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
+        g_signal_connect(G_OBJECT (p->topgwin), "enter-notify-event",
+              G_CALLBACK (panel_enter), p);    
+        g_signal_connect (G_OBJECT (p->topgwin), "drag-motion",
+          (GCallback) panel_drag_motion, p);
+        gtk_drag_dest_set (p->topgwin, GTK_DEST_DEFAULT_MOTION, 
+              NULL, 0, 0);
+        gtk_drag_dest_set_track_motion(p->topgwin, TRUE);
+
+        if (p->edge == EDGE_BOTTOM) {
+            p->ah_dx = 0;
+            p->ah_dy = p->ah - p->height_when_hidden;
+        } else if (p->edge == EDGE_TOP) {
+            p->ah_dx = 0;
+            p->ah_dy = - (p->ah - p->height_when_hidden);
+        } else if (p->edge == EDGE_LEFT) {
+            p->ah_dx = p->aw - p->height_when_hidden;
+            p->ah_dy = 0;
+        } else if (p->edge == EDGE_RIGHT) {
+            p->ah_dx = - (p->aw - p->height_when_hidden);
+            p->ah_dy = 0;
+        }
+        
+        panel_enter(NULL, NULL, p);
+    }
     calculate_position(p);
     gdk_window_move_resize(p->topgwin->window, p->ax, p->ay, p->aw, p->ah);
     if (p->setstrut)
         panel_set_wm_strut(p);
-  
     RET();
 }
 
@@ -431,7 +513,7 @@ panel_parse_global(panel *p, FILE *fp)
 {
     line s;
     s.len = 256;
-    
+
     ENTER;
     while (get_line(fp, &s) != LINE_NONE) {
         if (s.type == LINE_VAR) {
@@ -457,6 +539,10 @@ panel_parse_global(panel *p, FILE *fp)
                 p->setstrut = str2num(bool_pair, s.t[1], 0);
             } else if (!g_ascii_strcasecmp(s.t[0], "RoundCorners")) {
                 p->round_corners = str2num(bool_pair, s.t[1], 0);
+            } else if (!g_ascii_strcasecmp(s.t[0], "autohide")) {
+                p->autohide = str2num(bool_pair, s.t[1], 0);
+            } else if (!g_ascii_strcasecmp(s.t[0], "heightWhenHidden")) {
+                p->height_when_hidden = atoi(s.t[1]);                
             } else if (!g_ascii_strcasecmp(s.t[0], "Transparent")) {
                 p->transparent = str2num(bool_pair, s.t[1], 0);
             } else if (!g_ascii_strcasecmp(s.t[0], "Alpha")) {
@@ -515,7 +601,7 @@ panel_parse_plugin(panel *p, FILE *fp)
     gchar *type = NULL;
     FILE *tmpfp;
     int expand , padding, border;
-    
+
     ENTER;
     s.len = 256;
     if (!(tmpfp = tmpfile())) {
@@ -554,9 +640,9 @@ panel_parse_plugin(panel *p, FILE *fp)
                         pno++;
                     } else if (s.type == LINE_BLOCK_END) {
                         pno--;
-                    } 
+                    }
                     fprintf(tmpfp, "%s\n", s.str);
-                }              
+                }
             } else {
                 ERR( "fbpanel: unknown block %s\n", s.t[0]);
                 goto error;
@@ -566,7 +652,7 @@ panel_parse_plugin(panel *p, FILE *fp)
             goto error;
         }
     }
-    
+
     if (!type || !(plug = plugin_load(type))) {
         ERR( "fbpanel: can't load %s plugin\n", type);
         goto error;
@@ -587,14 +673,14 @@ panel_parse_plugin(panel *p, FILE *fp)
     p->plugins = g_list_append(p->plugins, plug);
     g_free(type);
     RET(1);
-    
+
  error:
     fclose(tmpfp);
     g_free(type);
     if (plug)
           plugin_put(plug);
     RET(0);
-    
+
 }
 
 
@@ -603,7 +689,7 @@ panel_start(panel *p, FILE *fp)
 {
     line s;
     long pos;
-    
+
     /* parse global section */
     ENTER;
     s.len = 256;
@@ -617,6 +703,9 @@ panel_start(panel *p, FILE *fp)
     p->setdocktype = 1;
     p->setstrut = 1;
     p->round_corners = 0;
+    p->autohide = 0;
+    p->visible = VISIBLE;
+    p->height_when_hidden = 2;
     p->transparent = 0;
     p->alpha = 127;
     p->tintcolor = 0xFFFFFFFF;
@@ -634,7 +723,7 @@ panel_start(panel *p, FILE *fp)
         RET(0);
     }
     pos = ftell(fp);
-    while (get_line_as_is(fp, &s) != LINE_NONE) 
+    while (get_line_as_is(fp, &s) != LINE_NONE)
         fprintf(pconf, "%s\n", s.str);
     fseek(fp, pos, SEEK_SET);
 
@@ -643,7 +732,7 @@ panel_start(panel *p, FILE *fp)
             ERR( "fbpanel: expecting Plugin section\n");
             RET(0);
         }
-        if (!panel_parse_plugin(p, fp)) 
+        if (!panel_parse_plugin(p, fp))
             RET(0);
     }
     gtk_widget_show_all(p->topgwin);
@@ -658,7 +747,7 @@ delete_plugin(gpointer data, gpointer udata)
     plugin_stop((plugin *)data);
     plugin_put((plugin *)data);
     RET();
-    
+
 }
 
 void panel_stop(panel *p)
@@ -668,7 +757,7 @@ void panel_stop(panel *p)
     g_list_foreach(p->plugins, delete_plugin, NULL);
     g_list_free(p->plugins);
     p->plugins = NULL;
- 
+
     XSelectInput (GDK_DISPLAY(), GDK_ROOT_WINDOW(), NoEventMask);
     gdk_window_remove_filter(gdk_get_default_root_window (), (GdkFilterFunc)panel_event_filter, p);
     gtk_widget_destroy(p->topgwin);
@@ -722,7 +811,7 @@ open_profile(gchar *profile)
     }
     //ERR("Can't load %s\n", fname);
     g_free(fname);
-    
+
     /* check private configuration directory */
     fname = g_strdup_printf("%s/share/fbpanel/%s", PREFIX, profile);
     fp = fopen(fname, "r");
@@ -766,7 +855,13 @@ main(int argc, char *argv[], char *env[])
     FILE *pfp; /* current profile FP */
     GdkPixbuf* icon;
     ENTER;
-    //printf("sizeof(gulong)=%d\n", sizeof(gulong));
+#if 0
+    printf("sizeof(gulong)=%d\n", sizeof(gulong));
+    printf("sizeof(Atom)=%d\n", sizeof(Atom));
+    printf("sizeof(Window)=%d\n", sizeof(Window));
+    printf("sizeof(gboolean)=%d\n", sizeof(gboolean));
+    RET(1);
+#endif
     setlocale(LC_CTYPE, "");
     gtk_set_locale();
     gtk_init(&argc, &argv);
@@ -813,7 +908,7 @@ main(int argc, char *argv[], char *env[])
     }
 
     gtk_icon_theme_append_search_path(gtk_icon_theme_get_default(),
-	IMGPREFIX);       
+    IMGPREFIX);
     signal(SIGUSR1, sig_usr);
     do {
         if (!(pfp = open_profile(cprofile)))
@@ -831,7 +926,7 @@ main(int argc, char *argv[], char *env[])
         panel_stop(p);
         g_free(p);
     } while (force_quit == 0);
-    
+
     exit(0);
 }
 
