@@ -17,6 +17,7 @@ enum { WIDTH_NONE, WIDTH_REQUEST, WIDTH_PIXEL, WIDTH_PERCENT };
 enum { HEIGHT_NONE, HEIGHT_PIXEL, HEIGHT_REQUEST };
 enum { ORIENT_NONE, ORIENT_VERT, ORIENT_HORIZ };
 enum { POS_NONE, POS_START, POS_END };
+enum { HIDDEN, TO_HIDE, VISIBLE };
 
 #define PANEL_HEIGHT_DEFAULT  26
 #define PANEL_HEIGHT_MAX      200
@@ -38,7 +39,7 @@ typedef struct {
     int alpha;
     guint32 tintcolor;
     GdkColor gtintcolor;
-    
+
     int ax, ay, aw, ah;  /* prefferd allocation of a panel */
     int cx, cy, cw, ch;  /* current allocation (as reported by configure event) allocation */
     int allign, edge, margin;
@@ -46,25 +47,33 @@ typedef struct {
     int widthtype, width;
     int heighttype, height;
 
-    int self_destroy : 1;
-    int setdocktype : 1;
-    int setstrut : 1;
-    int round_corners : 1;
-    int transparent : 1;
-    int spacing;
+    guint self_destroy : 1;
+    guint setdocktype : 1;
+    guint setstrut : 1;
+    guint round_corners : 1;
+    guint transparent : 1;
+
+    guint autohide : 1;
+    guint visible : 2;
+    int ah_dx, ah_dy; // autohide shift for x and y
+    int height_when_hidden;
+    guint hide_tout;
     
-    int desknum;
-    int curdesk;
+    int spacing;
+
+    guint desknum;
+    guint curdesk;
     guint32 *workarea;
     int wa_len;
+    guint style_timer;
 
     int plug_num;
     GList *plugins;
-    
+
 } panel;
 
 
-typedef struct { 
+typedef struct {
     unsigned int modal : 1;
     unsigned int sticky : 1;
     unsigned int maximized_vert : 1;
@@ -78,7 +87,7 @@ typedef struct {
     unsigned int below : 1;
 } net_wm_state;
 
-typedef struct { 
+typedef struct {
     unsigned int desktop : 1;
     unsigned int dock : 1;
     unsigned int toolbar : 1;
@@ -134,7 +143,7 @@ extern Atom a_NET_WM_WINDOW_TYPE_UTILITY;
 extern Atom a_NET_WM_WINDOW_TYPE_SPLASH;
 extern Atom a_NET_WM_WINDOW_TYPE_DIALOG;
 extern Atom a_NET_WM_WINDOW_TYPE_NORMAL;
-          
+
 extern Atom a_NET_WM_DESKTOP;
 extern Atom a_NET_WM_NAME;
 extern Atom a_NET_WM_VISIBLE_NAME;
