@@ -14,28 +14,21 @@
 //#define DEBUG
 #include "dbg.h"
 
-/*
- * SuxPanel version 0.1
- * Copyright (c) 2003 Leandro Pereira <leandro@linuxmag.com.br>
- */
-
-/*
- * menu style code was taken from suxpanel
- */
-
 
 static const char desktop_ent[] = "Desktop Entry";
 static const gchar app_dir_name[] = "applications";
 static GHashTable *ht;
+static GtkWidget *read_submenu(plugin *p, gboolean as_item);
 
 typedef struct {
     gchar *name;
     gchar *icon;
+    gchar *local_name;
     GtkWidget *menu;
 } cat_info;
 
 static cat_info main_cats[] = {
-    { "AudioVideo", "gnome-multimedia" },
+    { "AudioVideo", "gnome-multimedia", "Audio & Video" },
     { "Development","gnome-devel" },
     { "Education",  "gnome-applications" },
     { "Game",       "gnome-joystick" },
@@ -61,9 +54,11 @@ menu_destructor(plugin *p)
 
     ENTER;
     g_signal_handler_disconnect(G_OBJECT(m->bg), m->handler_id);
-    DBG("here\n");
-    gtk_widget_destroy(m->menu);
-    DBG("here\n");
+    if (m->menu) {
+        DBG("destroy(m->menu)\n");
+        gtk_widget_destroy(m->menu);
+    }
+    DBG("destroy(m->box)\n");
     gtk_widget_destroy(m->box);
     DBG("here\n");
     g_free(m);
@@ -75,7 +70,7 @@ spawn_app(GtkWidget *widget, gpointer data)
 {
     GError *error = NULL;
 
-    ENTER;    
+    ENTER;
     if (data) {
         if (! g_spawn_command_line_async(data, &error) ) {
             ERR("can't spawn %s\nError is %s\n", (char *)data, error->message);
@@ -85,6 +80,24 @@ spawn_app(GtkWidget *widget, gpointer data)
     RET();
 }
 
+static gint
+_menu_shell_insert_sorted(GtkMenuShell *menu_shell, GtkWidget *mi, const gchar *name)
+{
+    GList *items;
+    gint i;
+    gchar *cmpname;
+
+    //TRACE("dummy");
+
+    items = gtk_container_get_children(GTK_CONTAINER(menu_shell));
+    for(i=0; items; items=items->next, i++)  {
+        cmpname = (gchar *)g_object_get_data(G_OBJECT(items->data), "item-name");
+        if(cmpname && g_ascii_strcasecmp(name, cmpname) < 0)
+            break;
+    }
+    gtk_menu_shell_insert(menu_shell, mi, i);
+    return i;
+}
 
 static void
 do_app_dir(plugin *p, const gchar *path)
@@ -119,13 +132,14 @@ do_app_dir(plugin *p, const gchar *path)
         if (!(cats = g_key_file_get_string_list(file, desktop_ent, "Categories", NULL, NULL)))
             continue;
         if (!(exec = g_key_file_get_string(file, desktop_ent, "Exec", NULL)))
-            continue;
+            goto free_cats;
         while ((dot = strchr(exec, '%'))) {
             if (dot[1] != '\0')
                 dot[0] = dot[1] = ' ';
         }
         DBG("exec: %s\n", exec);
-        title = g_key_file_get_locale_string(file, desktop_ent, "Name", NULL, NULL);
+        if (!(title = g_key_file_get_locale_string(file, desktop_ent, "Name", NULL, NULL)))
+            goto free_exec;
         DBG("title: %s\n", title);
         icon = g_key_file_get_string(file, desktop_ent, "Icon", NULL);
         if (icon) {
@@ -133,29 +147,32 @@ do_app_dir(plugin *p, const gchar *path)
             if(icon[0] !='/' && dot )
                 *dot = '\0';
         }
-
         DBG("icon: %s\n", icon);
         for (tmp = cats; *tmp; tmp++) {
             GtkWidget **menu, *mi;
-            
+
             DBG("cat: %s\n", *tmp);
             if (!(menu = g_hash_table_lookup(ht, tmp[0])))
                 continue;
 
-            mi = gtk_image_menu_item_new_with_label(title ? title : exec);
+            mi = gtk_image_menu_item_new_with_label(title);
             gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi),
-                  fb_image_new_from_icon_file(icon, icon, 20, 20, TRUE));
-            g_signal_connect(G_OBJECT(mi), "activate", (GCallback)spawn_app, exec);
-            //g_object_set_data_full(G_OBJECT(mi), "activate", exec, g_free);
+                  gtk_fbimage_new(icon, icon, 22, 22, TRUE));
+            g_signal_connect(G_OBJECT(mi), "activate", (GCallback)spawn_app,g_strdup(exec));
             if (!(*menu))
                 *menu = gtk_menu_new();
-            gtk_menu_shell_prepend(GTK_MENU_SHELL(*menu), mi);
+            g_object_set_data_full(G_OBJECT(mi), "item-name", title, g_free);
+            _menu_shell_insert_sorted(GTK_MENU_SHELL(*menu), mi, title);
+            //gtk_menu_shell_prepend(GTK_MENU_SHELL(*menu), mi);
             gtk_widget_show_all(mi);
             DBG("added =======================================\n");
+            break;
         }
         g_free(icon);
-        g_free(title);
-        //g_free(exec);
+    free_exec:
+        g_free(exec);
+    free_cats:
+        g_strfreev(cats);
     }
     g_chdir(cwd);
     g_free(cwd);
@@ -170,7 +187,7 @@ make_fdo_menu(plugin *p, GtkWidget *menu)
     int i;
     gchar *path;
     menup *m = (menup *)p->priv;
-    
+
     ENTER;
     ht = g_hash_table_new(g_str_hash, g_str_equal);
     for (i = 0; i < G_N_ELEMENTS(main_cats); i++) {
@@ -179,7 +196,7 @@ make_fdo_menu(plugin *p, GtkWidget *menu)
         if (g_hash_table_lookup(ht, &main_cats[i].name))
             DBG("%s not found\n", main_cats[i].name);
     }
-    
+
     for (i = 0; i < g_strv_length((gchar **)sys_dirs); ++i)    {
         path = g_build_filename(sys_dirs[i], app_dir_name, NULL );
         do_app_dir(p, path);
@@ -190,13 +207,14 @@ make_fdo_menu(plugin *p, GtkWidget *menu)
     g_free(path);
     //build menu
     for (i = 0; i < G_N_ELEMENTS(main_cats); i++) {
-        GtkWidget *mi, *img;
-    
+        GtkWidget *mi;
+        gchar *name;
+
         if (main_cats[i].menu) {
-            mi = gtk_image_menu_item_new_with_label(main_cats[i].name);
-            img = fb_image_new_from_icon_file(main_cats[i].icon, NULL, m->iconsize, m->iconsize, TRUE);
-            gtk_widget_show(img);
-            gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+            name = main_cats[i].local_name ? main_cats[i].local_name : main_cats[i].name;
+            mi = gtk_image_menu_item_new_with_label(name);
+            gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi),
+                gtk_fbimage_new(main_cats[i].icon, NULL, m->iconsize, m->iconsize, TRUE));
             gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), main_cats[i].menu);
             gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
             gtk_widget_show_all(mi);
@@ -211,55 +229,42 @@ make_fdo_menu(plugin *p, GtkWidget *menu)
 static void
 run_command(GtkWidget *widget, void (*cmd)(void))
 {
-    ENTER;    
+    ENTER;
     cmd();
     RET();
 }
 
-#if 0
-static void
-menu_pos(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *widget)
+static gboolean
+delayed_menu_creation(plugin *p)
 {
-    int ox, oy, w, h;
-    plugin *p;
-    
+    menup *m;
+
     ENTER;
-    p = g_object_get_data(G_OBJECT(widget), "plugin");
-    gdk_window_get_origin(widget->window, &ox, &oy);
-    w = GTK_WIDGET(menu)->requisition.width;
-    h = GTK_WIDGET(menu)->requisition.height;
-    if (p->panel->orientation == ORIENT_HORIZ) {
-        *x = ox;
-        if (*x + w > gdk_screen_width())
-            *x = ox + widget->allocation.width - w;
-        *y = oy - h;
-        if (*y < 0)
-            *y = oy + widget->allocation.height;
-    } else {
-        *x = ox + widget->allocation.width;
-        if (*x > gdk_screen_width())
-            *x = ox - w;
-        *y = oy;
-        if (*y + h >  gdk_screen_height())
-            *y = oy + widget->allocation.height - h;        
+    m = (menup *)p->priv;
+    if (!m->menu) {
+        fseek(p->fp, 0, SEEK_SET);
+        read_submenu(p, TRUE);
     }
-    DBG("widget: x,y=%d,%d  w,h=%d,%d\n", ox, oy,
-          widget->allocation.width, widget->allocation.height );
-    DBG("w-h %d %d\n", w, h);
-    *push_in = TRUE;
-    RET();
+    RET(FALSE); /* run once */
 }
-#endif
 
 static gboolean
-my_button_pressed(GtkWidget *widget, GdkEventButton *event, GtkMenu *menu)
+my_button_pressed(GtkWidget *widget, GdkEventButton *event, plugin *p)
 {
-    ENTER;  
+    menup *m;
+
+    ENTER;
+    m = (menup *)p->priv;
     if ((event->type == GDK_BUTTON_PRESS)
           && (event->x >=0 && event->x < widget->allocation.width)
           && (event->y >=0 && event->y < widget->allocation.height)) {
-        gtk_menu_popup(menu,
-              NULL, NULL, (GtkMenuPositionFunc)menu_pos, widget, 
+        if (!m->menu) {
+            DBG("building menu\n");
+            fseek(p->fp, 0, SEEK_SET);
+            read_submenu(p, TRUE);
+        }
+        gtk_menu_popup(GTK_MENU(m->menu),
+              NULL, NULL, (GtkMenuPositionFunc)menu_pos, widget,
               event->button, event->time);
     }
     RET(TRUE);
@@ -271,10 +276,9 @@ make_button(plugin *p, gchar *iname, gchar *fname, gchar *name, GtkWidget *menu)
 {
     int w, h;
     menup *m;
-    
+
     ENTER;
     m = (menup *)p->priv;
-    m->menu = menu;
     if (p->panel->orientation == ORIENT_HORIZ) {
         w = 10000;
         h = p->panel->ah;
@@ -284,16 +288,16 @@ make_button(plugin *p, gchar *iname, gchar *fname, gchar *name, GtkWidget *menu)
     }
     DBG("iname=%s\n", iname);
     m->bg = fb_button_new_from_icon_file_with_label(iname, fname, w, h, 0x702020, TRUE, name);
-    gtk_widget_show(m->bg);  
+    gtk_widget_show(m->bg);
     gtk_box_pack_start(GTK_BOX(m->box), m->bg, FALSE, FALSE, 0);
-    if (p->panel->transparent) 
+    if (p->panel->transparent)
         gtk_bgbox_set_background(m->bg, BG_ROOT, p->panel->tintcolor, p->panel->alpha);
-    
+
 
     m->handler_id = g_signal_connect (G_OBJECT (m->bg), "button-press-event",
-          G_CALLBACK (my_button_pressed), menu);
+          G_CALLBACK (my_button_pressed), p);
     g_object_set_data(G_OBJECT(m->bg), "plugin", p);
-   
+
     RET(m->bg);
 }
 
@@ -305,14 +309,14 @@ read_item(plugin *p)
     GtkWidget *item;
     menup *m = (menup *)p->priv;
     void (*cmd)(void);
-    
+
     ENTER;
     s.len = 256;
     name = fname = action = iname = NULL;
     cmd = NULL;
     while (get_line(p->fp, &s) != LINE_BLOCK_END) {
         if (s.type == LINE_VAR) {
-            if (!g_ascii_strcasecmp(s.t[0], "image")) 
+            if (!g_ascii_strcasecmp(s.t[0], "image"))
                 fname = expand_tilda(s.t[1]);
             else if (!g_ascii_strcasecmp(s.t[0], "name"))
                 name = g_strdup(s.t[1]);
@@ -322,7 +326,7 @@ read_item(plugin *p)
                 action = expand_tilda(s.t[1]);
             else if (!g_ascii_strcasecmp(s.t[0], "command")) {
                 command *tmp;
-                
+
                 for (tmp = commands; tmp->name; tmp++) {
                     if (!g_ascii_strcasecmp(s.t[1], tmp->name)) {
                         cmd = tmp->cmd;
@@ -333,7 +337,7 @@ read_item(plugin *p)
                 ERR( "menu/item: unknown var %s\n", s.t[0]);
                 goto error;
             }
-        } 
+        }
     }
     /* menu button */
     item = gtk_image_menu_item_new_with_label(name ? name : "");
@@ -341,11 +345,8 @@ read_item(plugin *p)
     if (name)
         g_free(name);
     if (fname || iname) {
-        GtkWidget *img;
-        
-        img = fb_image_new_from_icon_file(iname, fname, m->iconsize, m->iconsize, TRUE);
-        gtk_widget_show(img);
-        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), img);
+        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
+            gtk_fbimage_new(iname, fname, m->iconsize, m->iconsize, TRUE));
         g_free(fname);
         g_free(iname);
     }
@@ -383,7 +384,7 @@ static void
 read_system_menu(plugin *p, GtkWidget *menu)
 {
     line s;
-    
+
     ENTER;
     s.len = 256;
     while (get_line(p->fp, &s) != LINE_BLOCK_END) ;
@@ -398,7 +399,7 @@ read_include(plugin *p)
     line s;
     menup *m = (menup *)p->priv;
     FILE *fp = NULL;
-    
+
     ENTER;
     s.len = 256;
     name = NULL;
@@ -431,17 +432,19 @@ read_submenu(plugin *p, gboolean as_item)
     gchar name[256], *fname, *iname;
     menup *m = (menup *)p->priv;
 
-    
+
     ENTER;
     s.len = 256;
     menu = gtk_menu_new ();
     gtk_container_set_border_width(GTK_CONTAINER(menu), 0);
-  
+
     iname = fname = NULL;
     name[0] = 0;
     DBG("here\n");
-    while (get_line(p->fp, &s) != LINE_BLOCK_END) {       
+    while (get_line(p->fp, &s) != LINE_BLOCK_END) {
         if (s.type == LINE_BLOCK_START) {
+            if (!as_item)
+                break;
             mi = NULL;
             if (!g_ascii_strcasecmp(s.t[0], "item")) {
                 mi = read_item(p);
@@ -467,7 +470,7 @@ read_submenu(plugin *p, gboolean as_item)
             gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
         } else if (s.type == LINE_VAR) {
             DBG("here\n");
-            if (!g_ascii_strcasecmp(s.t[0], "image")) 
+            if (!g_ascii_strcasecmp(s.t[0], "image"))
                 fname = expand_tilda(s.t[1]);
             else if (!g_ascii_strcasecmp(s.t[0], "name"))
                 strcpy(name, s.t[1]);
@@ -501,6 +504,7 @@ read_submenu(plugin *p, gboolean as_item)
             g_free(fname);
         }
         gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
+        m->menu = menu;
         RET(mi);
     } else {
         mi = make_button(p, iname, fname, name, menu);
@@ -508,9 +512,10 @@ read_submenu(plugin *p, gboolean as_item)
             g_free(fname);
         RET(mi);
     }
-    
+
  error:
     // FIXME: we need to recursivly destroy all child menus and their items
+    DBG("destroy(menu)\n");
     gtk_widget_destroy(menu);
     g_free(fname);
     g_free(name);
@@ -518,6 +523,20 @@ read_submenu(plugin *p, gboolean as_item)
 }
 
 
+static void
+menu_icon_theme_changed(GtkIconTheme *icon_theme, plugin *p)
+{
+    menup *m;
+
+    ENTER;
+    m = (menup *)p->priv;
+    if (m->menu) {
+        DBG("destroy(m->menu)\n");
+        gtk_widget_destroy(m->menu);
+    }
+    m->menu = NULL;
+    g_timeout_add(3000, (GSourceFunc) delayed_menu_creation, p);
+}
 
 
 static int
@@ -529,18 +548,7 @@ menu_constructor(plugin *p)
     m = g_new0(menup, 1);
     g_return_val_if_fail(m != NULL, 0);
     p->priv = m;
-
-    //gtk_rc_parse_string(menu_rc);
-#if 0
-    if  (p->panel->orientation == ORIENT_HORIZ) 
-        m->paneliconsize = p->panel->ah
-            - 2* GTK_WIDGET(p->panel->box)->style->ythickness;
-    else
-        m->paneliconsize = p->panel->aw
-            - 2* GTK_WIDGET(p->panel->box)->style->xthickness;
-#endif
-    m->iconsize = 24;       
-
+    m->iconsize = 22;
     m->box = gtk_hbox_new(FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(m->box), 0);
     gtk_container_add(GTK_CONTAINER(p->pwid), m->box);
@@ -549,6 +557,9 @@ menu_constructor(plugin *p)
         ERR("menu: plugin init failed\n");
         goto error;
     }
+    g_timeout_add(3000, (GSourceFunc) delayed_menu_creation, p);
+    g_signal_connect (G_OBJECT(gtk_icon_theme_get_default()),
+        "changed", (GCallback) menu_icon_theme_changed, p);
     RET(1);
 
  error:
