@@ -43,7 +43,7 @@ typedef struct _task{
     XClassHint ch;
     int pos_x;
     int width;
-    int desktop;
+    guint desktop;
     net_wm_state nws;
     net_wm_window_type nwwt;
     guint flash_timeout;
@@ -74,7 +74,7 @@ typedef struct _taskbar{
     int req_width;
     int hbox_width;
     int spacing;
-    int cur_desk;
+    guint cur_desk;
     task *focused;
     task *ptk;
     task *menutask;
@@ -892,6 +892,14 @@ tk_build_gui(taskbar *tb, task *tk)
     RET();
 }
 
+/* tell to remove element with zero refcount */
+static gboolean
+tb_remove_all_tasks(Window *win, task *t)
+{
+    g_free(t);
+    return TRUE;
+}
+
 /* tell to remove element with zero refcount */
 static gboolean
 tb_remove_stale_tasks(Window *win, task *tk, gpointer data)
@@ -941,7 +949,7 @@ tb_net_client_list(GtkWidget *widget, taskbar *tb)
             tb->num_tasks++;
             tk->win = tb->wins[i];
             tk->tb = tb;
-            tk->iconified = (get_wm_state(tk->win) == IconicState);
+            tk->iconified = nws.hidden;
             tk->desktop = get_net_wm_desktop(tk->win);
             tk->nws = nws;
             tk->nwwt = nwwt;
@@ -1076,21 +1084,12 @@ tb_propertynotify(taskbar *tb, XEvent *ev)
             DBG("NET_WM_DESKTOP\n");
 	    tk->desktop = get_net_wm_desktop(win);
 	    tb_display(tb);	
-	}  else if (at == XA_WM_NAME) {
+	} else if (at == XA_WM_NAME) {
             DBG("WM_NAME\n");
             if (!tb->icons_only) {
                 tk_get_names(tk);
                 tk_set_names(tk);
             }
-	}  else if (at == XA_WM_CLASS) {
-            DBG("WM_CLASS\n");
-	} else if (at == a_WM_STATE)    {
-            DBG("WM_STATE\n");
-	    /* iconified state changed? */
-            if (!tb->icons_only) {
-                tk->iconified = (get_wm_state (tk->win) == IconicState);
-                tk_set_names(tk);
-            }
 	} else if (at == XA_WM_HINTS)	{
 	    /* some windows set their WM_HINTS icon after mapping */
 	    DBG("XA_WM_HINTS\n");
@@ -1113,7 +1112,10 @@ tb_propertynotify(taskbar *tb, XEvent *ev)
             if (!accept_net_wm_state(&nws, tb->accept_skip_pager)) {
 		del_task(tb, tk, 1);
 		tb_display(tb);
-	    }
+	    } else if (!tb->icons_only){
+                tk->iconified = nws.hidden;
+                tk_set_names(tk);
+            }
 	} else if (at == a_NET_WM_ICON) {
 	    DBG("_NET_WM_ICON\n");
             DBG("#0 %d\n", GDK_IS_PIXBUF (tk->pixbuf));
@@ -1378,7 +1380,8 @@ taskbar_destructor(plugin *p)
     g_signal_handlers_disconnect_by_func(G_OBJECT (fbev), tb_net_number_of_desktops, tb);
     g_signal_handlers_disconnect_by_func(G_OBJECT (fbev), tb_net_client_list, tb);   
     gdk_window_remove_filter(NULL, (GdkFilterFunc)tb_event_filter, tb );
-    g_hash_table_destroy(tb->task_list); 
+    g_hash_table_foreach_remove(tb->task_list, (GHRFunc) tb_remove_all_tasks, NULL);
+    g_hash_table_destroy(tb->task_list);
     gtk_widget_destroy(tb->bar);
     gtk_widget_destroy(tb->menu);
   
